
% This is a code for constrained and unconstrained single objective
% optimization: Authors Md. Asafoudulla Asaf and Tapabrata Ray

% Generates a constraint sequence from 1 to number of constraints
% population is divided equal to the number of constraints and 
% each subpopulation is assigned a particular constraint sequence 
% to evaluate, the constraint sequences are generated following a ring
% topology e.g. (g1, g2, g3), (g2,g3,g1) and (g3,g1,g2) 
% Archive is used and the size is normaly double the size of popualation
% Uses DE/rand/bin crossover to generate child solution

% A gradient is used to improve the child solution and the epsilon based
% comparison is made upto 1500 genereations. The treshold is decline following 
% threshold=threshold/gen 

function [pop]=DE_MS(p_name,def)

def.problem_name={p_name};
prob=load_problem_definition(def);
%%%%%%statistical variables%%%%%%
FE=zeros(def.Gen,1);
G_FE=zeros(def.Gen,prob.ng);
statlist=[];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gen=1;
[pop]=initalize_pop(def,prob);
[archive]=initalize_archive(def,prob);
%evaluate population
for i=1:def.active_pop
    [pop.f(i,:),pop.g(i,:)]=evaluate(pop.x(i,:),def,pop.constseq,'init');
    [FE(gen),G_FE(gen,:)]=Compute_FE(FE(gen),G_FE(gen,:),pop.f(i,:),pop.g(i,:),pop.constseq);
end
%do the implicit hardness ranking on constraints
[pop]=FindConstraintSequence(pop);
%use the given constrint sequence
pop.constseq=1:prob.ng;
%Build the subpopulation based on the hardness constraint sequence
[pop]=BuildPopulation(pop);
%Evaluate the subpopulation on particular constraint sequence
if(def.strategy==2)
    [pop]=evalpop(pop);
end
% Apply the sequence sorting for feasible population
if(~isempty(pop.feas))
    id=pop.feas;
    [rank]=SequenceSort(pop.f,pop.g,id,def.strategy); % 1 for sequence sort of a feasible single objective problem
    pop.x(pop.feas',:)=pop.x(rank',:);
    pop.f(pop.feas',:)  =pop.f(rank',:);
    pop.g(pop.feas',:)=pop.g(rank',:);
end
pop.rank(pop.feas)=1:size(pop.feas,2);
%Apply the sequence sorting for the subpopulation
infeasible=[];
for i=1:size(pop.sp,2)
    if(~isempty(pop.sp{i}))
        id=pop.sp{i};
        [rank]=SequenceSort(pop.f,pop.g,id,def.strategy);
        pop.x(pop.sp{i}',:)=pop.x(rank',:);
        pop.f(pop.sp{i}',:)  =pop.f(rank',:);
        pop.g(pop.sp{i}',:)=pop.g(rank',:);
        infeasible=[infeasible id];
    end
end
%Update the individuals ranking
rank=[pop.feas infeasible];
pop.rank(rank)=1:def.active_pop;
% initiate best stat
beststat.f=pop.f(1,:);
beststat.g=pop.g(1,:);
beststat.x=pop.x(1,:);
if(pop.feasible(1)==1)
    beststat.subpop=1;
else
    beststat.subpop=2;
end
beststat.FE=sum(FE);
beststat.GFE=sum(G_FE);
beststat.CR=pop.CR(1);
beststat.F=pop.F(1);
%save the initial statistics
savestat(gen,pop,beststat,'initial',def,prob);
gen=2;
Th=sum(pop.g(randi(def.active_pop),:));
while(gen<=def.Gen)
    %evolve feasible population
    if(~isempty(pop.feas))
        id=pop.feas;subpop=1;ndpop=[];
        p1=randperm(size(id,2));
        for i=1:size(id,2)
            p2=randi(def.active_pop);
            if(rand<0.05)
                p3=randi(def.archive);
                [childpop,FE(gen),G_FE(gen,:)]=DE_evolve(FE(gen),G_FE(gen,:),pop,archive,pop.constseq,id(i),id(p1(i)),p2,p3,def,prob,'archive');
            else
                p3=randi(def.active_pop);
                [childpop,FE(gen),G_FE(gen,:)]=DE_evolve(FE(gen),G_FE(gen,:),pop,archive,pop.constseq,id(i),id(p1(i)),p2,p3,def,prob,'');
            end
            if(prob.nf==1)
                [pop,archive,beststat]=selection(beststat,pop.constseq,pop,archive,id(i),childpop,def,subpop,Th,prob);
            else
                ndpop.f(i,:)=childpop.f;
                ndpop.g(i,:)=childpop.g;
                ndpop.x(i,:)=childpop.x;
            end
        end
        if(~isempty(ndpop))
            [pop]=ndreduced(ndpop,pop,id);
        end
    end
    for j=1:size(pop.sp,2)
        if(~isempty(pop.sp{j}))
            id=pop.sp{j};subpop=j;
            p1=randperm(size(id,2));
            for i=1:size(id,2)
                p2=randi(def.active_pop);
                if(rand<0.05)
                    p3=randi(def.archive);
                    [childpop,FE(gen),G_FE(gen,:)]=DE_evolve(FE(gen),G_FE(gen,:),pop,archive,pop.constseq,id(i),id(p1(i)),p2,p3,def,prob,'archive');
                else
                    p3=randi(def.active_pop);
                    [childpop,FE(gen),G_FE(gen,:)]=DE_evolve(FE(gen),G_FE(gen,:),pop,archive,pop.constseq,id(i),id(p1(i)),p2,p3,def,prob,'');
                end
                [pop,archive,beststat]=selection(beststat,pop.subsequence(j,:),pop,archive,id(i),childpop,def,subpop,Th,prob);
            end
        end
    end
    %population migration from each subpopulation to feasible region if it
    %belongs to
    [pop,feas]=PopulationMigration(pop);
    pop.feas=[pop.feas,feas];
    %Infeasible solutions ranking
    infeasible=[];
    for i=1:size(pop.sp,2)
        if(~isempty(pop.sp{i}))
            id=pop.sp{i};
            infeasible=[infeasible id];
        end
    end
    %individual ranking in the population based on fitness
    rank=[pop.feas infeasible];
    pop.rank(rank)=1:def.active_pop;
    %save the statistics
    beststat.FE=sum(FE);
    beststat.GFE=sum(G_FE);
    savestat(gen,pop,beststat,'',def,prob);
    %display the statistics
    disp('FE');
    disp(sum(FE));
    disp('Function value:');
    disp(beststat.f);
    disp('Constraints:');
    disp(beststat.g);
    disp('GFE:');
    disp(sum(sum(G_FE))/3);
    %show the current progress plot
    statlist=show_progress(gen,pop,statlist,1);
    %if(beststat.f(1)-(op)<=0.0001 && sum(beststat.g)==0)
    %   break;
    %end
    %Update the epsilon threshold 
    if(gen<=1500)
        Th=Th/gen;
    else
        Th=0;
    end
    %Update the generations
    gen=gen+1;
end
savestat((gen-1),pop,beststat,'final',def,prob);
return







